@@ -250,6 +250,89 @@ class Concatenate1D(Layer):
             return (input_shape[0][0], None, input_shape[0][2])
 
 
+class BatchSplit(Layer):
+    """将第一维进行分割
+    主要是用于自行实现多卡数据并行。
+    """
+    def __init__(self, parts, **kwargs):
+        super(BatchSplit, self).__init__(**kwargs)
+        self.parts = parts
+
+    def compute_mask(self, inputs, mask=None):
+        if isinstance(mask, list):
+            return [o for i in mask for o in self.compute_mask(inputs, i)]
+
+        if mask is not None:
+            return self.call(mask)
+        elif np.ndim(self.parts) > 0:
+            return [None] * len(self.parts)
+        else:
+            return [None] * self.parts
+
+    def call(self, inputs):
+        if isinstance(inputs, list):
+            return [o for i in inputs for o in self.call(i)]
+
+        outputs = []
+
+        batch_size = K.shape(inputs)[0]
+        if np.ndim(self.parts) > 0:
+            batch_size = K.cast(batch_size, 'float64')
+            slices = [
+                K.cast(p * batch_size / sum(self.parts), 'int32')
+                for p in np.cumsum(self.parts).astype('float64')
+            ]
+        else:
+            stride = K.cast(
+                tf.math.ceil(batch_size / self.parts), K.dtype(batch_size)
+            )
+            slices = [stride * (i + 1) for i in range(self.parts)]
+
+        for i, _ in enumerate(slices):
+            if i == 0:
+                outputs.append(inputs[:slices[0]])
+            elif i == len(slices) - 1:
+                outputs.append(inputs[slices[-2]:])
+            else:
+                outputs.append(inputs[slices[i - 1]:slices[i]])
+
+        return outputs
+
+    def compute_output_shape(self, input_shape):
+        if isinstance(input_shape, list):
+            return [
+                o for i in input_shape for o in self.compute_output_shape(i)
+            ]
+
+        if np.ndim(self.parts) > 0:
+            return [input_shape] * len(self.parts)
+        else:
+            return [input_shape] * self.parts
+
+    def get_config(self):
+        config = {
+            'parts': self.parts,
+        }
+        base_config = super(BatchSplit, self).get_config()
+        return dict(list(base_config.items()) + list(config.items()))
+
+
+class BatchConcat(Layer):
+    """将第一维进行合并
+    主要是用于自行实现多卡数据并行。
+    """
+    def compute_mask(self, inputs, mask=None):
+        if isinstance(mask, list):
+            if all([m is not None for m in mask]):
+                return K.concatenate(mask, 0)
+
+    def call(self, inputs):
+        return K.concatenate(inputs, 0)
+
+    def compute_output_shape(self, input_shape):
+        return input_shape[0]
+
+
 class MultiHeadAttention(Layer):
     """多头注意力机制
     """
@@ -1179,6 +1262,7 @@ class MaximumEntropyMarkovModel(Layer):
 class GlobalPointer(Layer):
     """全局指针模块
     将序列的每个(start, end)作为整体来进行判断
+    参考：https://kexue.fm/archives/8373
     """
     def __init__(
         self,
@@ -1186,7 +1270,8 @@ class GlobalPointer(Layer):
         head_size,
         RoPE=True,
         use_bias=True,
-        kernel_initializer='glorot_uniform',
+        tril_mask=True,
+        kernel_initializer='lecun_normal',
         **kwargs
     ):
         super(GlobalPointer, self).__init__(**kwargs)
@@ -1194,6 +1279,7 @@ class GlobalPointer(Layer):
         self.head_size = head_size
         self.RoPE = RoPE
         self.use_bias = use_bias
+        self.tril_mask = tril_mask
         self.kernel_initializer = initializers.get(kernel_initializer)
 
     def build(self, input_shape):
@@ -1231,8 +1317,9 @@ class GlobalPointer(Layer):
         logits = sequence_masking(logits, mask, '-inf', 2)
         logits = sequence_masking(logits, mask, '-inf', 3)
         # 排除下三角
-        mask = tf.linalg.band_part(K.ones_like(logits), 0, -1)
-        logits = logits - (1 - mask) * K.infinity()
+        if self.tril_mask:
+            mask = tf.linalg.band_part(K.ones_like(logits), 0, -1)
+            logits = logits - (1 - mask) * K.infinity()
         # scale返回
         return logits / self.head_size**0.5
 
@@ -1245,6 +1332,7 @@ class GlobalPointer(Layer):
             'head_size': self.head_size,
             'RoPE': self.RoPE,
             'use_bias': self.use_bias,
+            'tril_mask': self.tril_mask,
             'kernel_initializer':
                 initializers.serialize(self.kernel_initializer),
         }
@@ -1252,6 +1340,54 @@ class GlobalPointer(Layer):
         return dict(list(base_config.items()) + list(config.items()))
 
 
+class EfficientGlobalPointer(GlobalPointer):
+    """更加参数高效的GlobalPointer
+    参考：https://kexue.fm/archives/8877
+    """
+    def build(self, input_shape):
+        self.dense_1 = Dense(
+            units=self.head_size * 2,
+            use_bias=self.use_bias,
+            kernel_initializer=self.kernel_initializer
+        )
+        self.dense_2 = Dense(
+            units=self.heads * 2,
+            use_bias=self.use_bias,
+            kernel_initializer=self.kernel_initializer
+        )
+        self.built = True
+
+    @recompute_grad
+    def call(self, inputs, mask=None):
+        # 输入变换
+        inputs = self.dense_1(inputs)
+        qw, kw = inputs[..., ::2], inputs[..., 1::2]
+        # RoPE编码
+        if self.RoPE:
+            pos = SinusoidalPositionEmbedding(self.head_size, 'zero')(inputs)
+            cos_pos = K.repeat_elements(pos[..., 1::2], 2, -1)
+            sin_pos = K.repeat_elements(pos[..., ::2], 2, -1)
+            qw2 = K.stack([-qw[..., 1::2], qw[..., ::2]], 3)
+            qw2 = K.reshape(qw2, K.shape(qw))
+            qw = qw * cos_pos + qw2 * sin_pos
+            kw2 = K.stack([-kw[..., 1::2], kw[..., ::2]], 3)
+            kw2 = K.reshape(kw2, K.shape(kw))
+            kw = kw * cos_pos + kw2 * sin_pos
+        # 计算内积
+        logits = tf.einsum('bmd,bnd->bmn', qw, kw) / self.head_size**0.5
+        bias = tf.einsum('bnh->bhn', self.dense_2(inputs)) / 2
+        logits = logits[:, None] + bias[:, ::2, None] + bias[:, 1::2, :, None]
+        # 排除padding
+        logits = sequence_masking(logits, mask, '-inf', 2)
+        logits = sequence_masking(logits, mask, '-inf', 3)
+        # 排除下三角
+        if self.tril_mask:
+            mask = tf.linalg.band_part(K.ones_like(logits), 0, -1)
+            logits = logits - (1 - mask) * K.infinity()
+        # 返回最终结果
+        return logits
+
+
 class Loss(Layer):
     """特殊的层，用来定义复杂loss
     """
@@ -1302,6 +1438,8 @@ custom_objects = {
     'BiasAdd': BiasAdd,
     'Scale': Scale,
     'Concatenate1D': Concatenate1D,
+    'BatchSplit': BatchSplit,
+    'BatchConcat': BatchConcat,
     'MultiHeadAttention': MultiHeadAttention,
     'LayerNormalization': LayerNormalization,
     'PositionEmbedding': PositionEmbedding,
@@ -1312,6 +1450,7 @@ custom_objects = {
     'ConditionalRandomField': ConditionalRandomField,
     'MaximumEntropyMarkovModel': MaximumEntropyMarkovModel,
     'GlobalPointer': GlobalPointer,
+    'EfficientGlobalPointer': EfficientGlobalPointer,
     'Loss': Loss,
 }
 
